#include "aliases.h"
#include "lexer.h"
#include "parser.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int main(int argc, char *argv[]) {
  if (argc < 2) {
    printf("NO FILE PROVIDED\n");

    return EXIT_FAILURE;
  }

  const char *filename = argv[1];

  FILE *fp = fopen(filename, "r");

  fseek(fp, 0, SEEK_END);

  u64 length = ftell(fp);

  fseek(fp, 0, SEEK_SET);

  char json[length + 1];
  memset(json, 0, length + 1);

  fread(json, sizeof(char), length, fp);

  fclose(fp);

  lexer_t *lexer = NULL;

  lexer_init(&lexer);
  if (!lexer) {
    return EXIT_FAILURE;
  }

  lex_result_t result = get_next_token(lexer, json);

  if (result.error.errno) {
    printf("%s\n", result.error.msg);
  } else {
    while (result.token.type != TK_NO_TOKEN) {
      print_token(result.token);

      parse_token(result.token);

      result = get_next_token(lexer, NULL);

      if (result.error.errno) {
        printf("%s\n", result.error.msg);
        break;
      }
    }
  }

  lexer_free(&lexer);

  return EXIT_SUCCESS;
}
